@@ -4,9 +4,9 @@
 *  This code is part of GLPK (GNU Linear Programming Kit).
 *
 *  Copyright (C) 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008,
-*  2009, 2010 Andrew Makhorin, Department for Applied Informatics,
-*  Moscow Aviation Institute, Moscow, Russia. All rights reserved.
-*  E-mail: <mao@gnu.org>.
+*  2009, 2010, 2011, 2013 Andrew Makhorin, Department for Applied
+*  Informatics, Moscow Aviation Institute, Moscow, Russia. All rights
+*  reserved. E-mail: <mao@gnu.org>.
 *
 *  GLPK is free software: you can redistribute it and/or modify it
 *  under the terms of the GNU General Public License as published by
@@ -25,9 +25,6 @@
 #ifndef GLPAPI_H
 #define GLPAPI_H
 
-#define GLP_PROB_DEFINED
-typedef struct glp_prob glp_prob;
-
 #include "glpk.h"
 #include "glpavl.h"
 #include "glpbfd.h"
@@ -40,7 +37,7 @@ typedef struct GLPAIJ GLPAIJ;
 
 struct glp_prob
 {     /* LP/MIP problem object */
-      int magic;
+      unsigned magic;
       /* magic value used for debugging */
       DMP *pool;
       /* memory pool to store problem object components */
@@ -300,6 +297,15 @@ int _glp_analyze_row(glp_prob *P, int len, const int ind[],
 void _glp_mpl_init_rand(glp_tran *tran, int seed);
 #endif
 
+#define glp_skpgen _glp_skpgen
+void glp_skpgen(int n, int r, int type, int v, int s, int a[],
+   int *b, int c[]);
+/* Pisinger's 0-1 single knapsack problem generator */
+
+#if 1 /* 28/V-2010 */
+int _glp_intopt1(glp_prob *P, const glp_iocp *parm);
+#endif
+
 #endif
 
 /* eof */
