@@ -4,9 +4,9 @@
 *  This code is part of GLPK (GNU Linear Programming Kit).
 *
 *  Copyright (C) 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008,
-*  2009, 2010 Andrew Makhorin, Department for Applied Informatics,
-*  Moscow Aviation Institute, Moscow, Russia. All rights reserved.
-*  E-mail: <mao@gnu.org>.
+*  2009, 2010, 2011, 2013 Andrew Makhorin, Department for Applied
+*  Informatics, Moscow Aviation Institute, Moscow, Russia. All rights
+*  reserved. E-mail: <mao@gnu.org>.
 *
 *  GLPK is free software: you can redistribute it and/or modify it
 *  under the terms of the GNU General Public License as published by
@@ -25,8 +25,6 @@
 #ifndef GLPIOS_H
 #define GLPIOS_H
 
-typedef struct glp_tree glp_tree;
-
 #include "glpapi.h"
 
 typedef struct IOSLOT IOSLOT;
@@ -165,10 +163,10 @@ struct glp_tree
       /* control parameters and statistics */
       const glp_iocp *parm;
       /* copy of control parameters passed to the solver */
-      xlong_t tm_beg;
+      glp_long tm_beg;
       /* starting time of the search, in seconds; the total time of the
          search is the difference between xtime() and tm_beg */
-      xlong_t tm_lag;
+      glp_long tm_lag;
       /* the most recent time, in seconds, at which the progress of the
          the search was displayed */
       int sol_cnt;
