@@ -4,9 +4,9 @@
 *  This code is part of GLPK (GNU Linear Programming Kit).
 *
 *  Copyright (C) 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008,
-*  2009, 2010 Andrew Makhorin, Department for Applied Informatics,
-*  Moscow Aviation Institute, Moscow, Russia. All rights reserved.
-*  E-mail: <mao@gnu.org>.
+*  2009, 2010, 2011, 2013 Andrew Makhorin, Department for Applied
+*  Informatics, Moscow Aviation Institute, Moscow, Russia. All rights
+*  reserved. E-mail: <mao@gnu.org>.
 *
 *  GLPK is free software: you can redistribute it and/or modify it
 *  under the terms of the GNU General Public License as published by
@@ -647,7 +647,7 @@ done: xfree(newtable);
 static int wclique(int _n, int w[], unsigned char _a[], int sol[])
 {     struct dsa _dsa, *dsa = &_dsa;
       int i, j, p, max_wt, max_nwt, wth, *used, *nwt, *pos;
-      xlong_t timer;
+      glp_long timer;
       n = _n;
       wt = &w[1];
       a = _a;
