@@ -4,9 +4,9 @@
 *  This code is part of GLPK (GNU Linear Programming Kit).
 *
 *  Copyright (C) 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008,
-*  2009, 2010 Andrew Makhorin, Department for Applied Informatics,
-*  Moscow Aviation Institute, Moscow, Russia. All rights reserved.
-*  E-mail: <mao@gnu.org>.
+*  2009, 2010, 2011, 2013 Andrew Makhorin, Department for Applied
+*  Informatics, Moscow Aviation Institute, Moscow, Russia. All rights
+*  reserved. E-mail: <mao@gnu.org>.
 *
 *  GLPK is free software: you can redistribute it and/or modify it
 *  under the terms of the GNU General Public License as published by
@@ -22,11 +22,8 @@
 *  along with GLPK. If not, see <http://www.gnu.org/licenses/>.
 ***********************************************************************/
 
-#define GLPSDF_H
-
-typedef struct glp_data glp_data;
-
 #include "glpapi.h"
+#include "glpsdf.h"
 
 struct glp_data
 {     /* plain data file */
@@ -90,7 +87,7 @@ void glp_sdf_error(glp_data *data, const char *fmt, ...)
       xvprintf(fmt, arg);
       va_end(arg);
       if (data->jump == NULL)
-         exit(EXIT_FAILURE);
+         xerror("");
       else
          longjmp(data->jump, 1);
       /* no return */
